package httpsdemo;

import java.util.concurrent.*;
import java.security.*;
import javax.net.ssl.*;
import org.apache.commons.lang3.*;
import org.apache.http.HttpHost;
import org.apache.http.auth.AuthScope;
import org.apache.http.auth.*;
import org.apache.http.client.*;
import org.apache.http.client.config.*;
import org.apache.http.config.*;
import org.apache.http.conn.socket.*;
import org.apache.http.conn.ssl.TrustAllStrategy;
import org.apache.http.conn.ssl.SSLConnectionSocketFactory;
import org.apache.http.impl.client.*;
import org.apache.http.impl.conn.PoolingHttpClientConnectionManager;
import org.apache.http.ssl.*;
import org.slf4j.Logger;

public class HttpClientUtils {
	private static String[] supportedProtocols = {"TLSv1.1"};
	private static String[] supportedCipherSuites = null;
	private static TrustStrategy trustStrategy = null;
	//private static TrustStrategy trustStrategy = TrustAllStrategy.INSTANCE; 
	private static HostnameVerifier hostnameVerifier = SSLConnectionSocketFactory.getDefaultHostnameVerifier();
	//private static HostnameVerifier hostnameVerifier = NoopHostnameVerifier.INSTANCE;
	private static Logger logger = LoggerUtils.getAppLogger();
	
    public static CloseableHttpClient createHttpClientWithCert(KeyStore trustStore, KeyStore keyStore, String keyStorePass,
                                                               int connMaxTotal,
                                                               int connDefaultMaxPerRoute,
                                                               int validateInactivityMillis,
                                                               int evictIdleConnectionsMillis,
                                                               String proxyHost,
                                                               int proxyPort,
                                                               String proxyUsername,
                                                               String proxyPassword) {
        SSLContext sslContext = null;
        SSLContextBuilder sslContextBuilder = null;
        try {
        	sslContextBuilder = SSLContexts.custom().loadTrustMaterial(trustStore, trustStrategy);
            if (keyStore!=null)
            	sslContextBuilder = sslContextBuilder.loadKeyMaterial(keyStore, keyStorePass.toCharArray());
            sslContext = sslContextBuilder.build();
        } catch (Exception e) {
        	logger.error("ssl context build failed", e);
            return null;
        }
        
        SSLConnectionSocketFactory sslsf = new SSLConnectionSocketFactory(
        		sslContext, supportedProtocols, supportedCipherSuites, hostnameVerifier);
        Registry<ConnectionSocketFactory> socketFactoryRegistry = RegistryBuilder.<ConnectionSocketFactory>create()
                .register("https", sslsf).register("http", PlainConnectionSocketFactory.INSTANCE).build();
        
        //ConnectionManager
        PoolingHttpClientConnectionManager connManager = new PoolingHttpClientConnectionManager(socketFactoryRegistry);
        SocketConfig socketConfig = SocketConfig.custom().setTcpNoDelay(true).build();
        connManager.setDefaultSocketConfig(socketConfig);
        connManager.setMaxTotal(connMaxTotal);
        connManager.setDefaultMaxPerRoute(connDefaultMaxPerRoute);
        connManager.setValidateAfterInactivity(validateInactivityMillis);
        
        //CookieStore
        CookieStore cookieStore = new BasicCookieStore();
        
        //RequestConfig
        RequestConfig requestConfig = RequestConfig.custom()
        		.setCookieSpec(CookieSpecs.DEFAULT)
                .setExpectContinueEnabled(true)
                .build();
        
        //Proxy
        HttpHost proxy = null;
        if (StringUtils.isNotEmpty(proxyHost)) {
            proxy = new HttpHost(proxyHost, proxyPort, "http");
        }
        CredentialsProvider credsProvider = new BasicCredentialsProvider();
        if (StringUtils.isNotEmpty(proxyUsername) && StringUtils.isNotEmpty(proxyPassword)) {
            credsProvider.setCredentials(new AuthScope(proxyHost, proxyPort),
                    new UsernamePasswordCredentials(proxyUsername, proxyPassword));
        }
        
        CloseableHttpClient httpclient = HttpClients.custom().setProxy(proxy)
                .setDefaultCredentialsProvider(credsProvider)
                .setConnectionManager(connManager)
                .setDefaultCookieStore(cookieStore)
                .setDefaultRequestConfig(requestConfig)
                .evictExpiredConnections()
                .evictIdleConnections(evictIdleConnectionsMillis, TimeUnit.MILLISECONDS)
                .build();
        return httpclient;
    }
}

